##################################################
#####     ERRORS                             #####
##################################################

## GENERAL ERRORS
class NotPythonModule(Exception): ...

## SET-UP ERRORS
class AppNotInitialised(Exception): ...
class ReSetupError(Exception): ...
class NotCompatibleOS(Exception): ...

## APP ERRORS
# Main Errors
class PasswordNotProvided(Exception): ...
class CommandNotFound(Exception): ...
class PasswordError(Exception): ...
class MissingArguments(Exception): ...

# WorkSpace Errors
class WorkSpaceAlreadyExists(Exception): ...
class WorkSpaceNotFound(Exception): ...

# Config Errors
class DirectoryNotFound(Exception): ...
class API_KEY_ERROR(Exception): ...
class InvalidSubCommand(Exception): ...
