########################################
#####  GLOBAL VARIABLES            #####
########################################

__version__ = "0.1.0" # Current Version of the project

# Long description
__description__ = "A project to help all fellow devs to not waste time in doing the boring and repetitive stuff"

# Other Metadata
__author__ = "Dtar380"
__license__ = "GPL v3.0"
__status__ = "4 - Beta"
